/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 */

#include "nsCOMPtr.h"
#include "nsString.h"
#include "nsIObserver.h"
#include "nsXPIDLString.h"
#include "nsIServiceManager.h"
#include "nsICategoryManager.h"
#include "nsAppStartupNotifier.h"
#include "nsISupportsPrimitives.h"


NS_IMPL_ISUPPORTS1(nsAppStartupNotifier, nsIAppStartupNotifier)

nsAppStartupNotifier::nsAppStartupNotifier()
{
	NS_INIT_ISUPPORTS();
}

nsAppStartupNotifier::~nsAppStartupNotifier()
{
}

nsresult nsAppStartupNotifier::Init()
{
	return NS_OK;
}

NS_IMETHODIMP nsAppStartupNotifier::NotifyStartupObservers()
{
	nsresult rv;

	// now initialize all startup listeners
	nsCOMPtr<nsICategoryManager> categoryManager =
				do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv);
	NS_ENSURE_SUCCESS(rv, rv);

	nsCOMPtr<nsISimpleEnumerator> enumerator;
	rv = categoryManager->EnumerateCategory(APPSTARTUP_CATEGORY,
								getter_AddRefs(enumerator));
	if (NS_FAILED(rv)) return rv;

	nsCOMPtr<nsISupports> entry;
	while (NS_SUCCEEDED(enumerator->GetNext(getter_AddRefs(entry)))) {
		nsCOMPtr<nsISupportsString> category = do_QueryInterface(entry, &rv);

		if (NS_SUCCEEDED(rv)) {
			nsXPIDLCString categoryEntry;
			rv = category->GetData(getter_Copies(categoryEntry));

			nsXPIDLCString contractId;
			categoryManager->GetCategoryEntry(APPSTARTUP_CATEGORY, categoryEntry,
										getter_Copies(contractId));

			if (NS_SUCCEEDED(rv)) {

				// creating a single instance
				// if you need a service, do a GetService() from the observer
				// this allows the instance to be released and the DLL unloaded,
				// if necessary
				nsCOMPtr<nsIObserver> startupObserver =
								do_CreateInstance(contractId, &rv);

				NS_ASSERTION(NS_SUCCEEDED(rv), "cannot create startup observer!\n");
				if (NS_SUCCEEDED(rv)) {
					rv = startupObserver->Observe(nsnull, APPSTARTUP_TOPIC, nsnull);
 
					// mainly for debugging if you want to know if your observer worked.
					NS_ASSERTION(NS_SUCCEEDED(rv), "Startup Observer failed!\n");
				}
			}
		}
	}

	return NS_OK;
}
